--==============================================================================
--
-- oUF_HealComm
--
-- Uses data from LibHealComm-4.0 to add incoming heal estimate bars onto units 
-- health bars.
--
-- * currently won't update the frame if max HP is unknown (ie, restricted to 
--   players/pets in your group that are in range), hides the bar for these
-- * can define frame.ignoreHealComm in layout to not have the bars appear on 
--   that frame
--
--=============================================================================
if not oUF then return end

--set color here
local color = {
    r = 0,
    g = 1,
    b = 0,
    a = .25,
}

local oUF_HealComm = {}

local healcomm = LibStub("LibHealComm-4.0")

--update a specific bar
local updateHealCommBar = function(frame, unit)
    local curHP = UnitHealth(frame.unit)
    local maxHP = UnitHealthMax(frame.unit)
    local percHP = curHP / maxHP

    local healAmount = healcomm:GetHealAmount(unit, healcomm.CASTED_HEALS) or 0
    local healModifier = healcomm:GetHealModifier(unit) or 1

    local incHeals = healAmount * healModifier
    
    if incHeals == 0 then
        frame.HealCommBar:Hide()
        return
    else
        frame.HealCommBar:Show()
    end

    local percInc = incHeals / maxHP

    frame.HealCommBar:SetWidth(percInc * frame.Health:GetWidth())
    frame.HealCommBar:SetPoint("LEFT", frame.Health, "LEFT", frame.Health:GetWidth() * percHP, 0)
end

--used by library callbacks, arguments should be list of units to update
local updateHealCommBars = function(...)
	for i = 1, select("#", ...) do
		local unit = select(i, ...)

        --search current oUF frames for this unit GUID
        for frame in pairs(oUF.units) do
            if unit == UnitGUID(oUF.units[frame].unit) then
                updateHealCommBar(oUF.units[frame],unit)
            end
        end
	end
end

local function hook(frame)
	if frame.ignoreHealComm then return end

    --create heal bar here and set initial values
	local hcb = CreateFrame"StatusBar"
	hcb:SetHeight(frame.Health:GetHeight()) -- same height as health bar
    hcb:SetWidth(0) --no initial width
	hcb:SetStatusBarTexture(frame.Health:GetStatusBarTexture():GetTexture())
    hcb:SetStatusBarColor(color.r, color.g, color.b, color.a)
    hcb:SetParent(frame)
    hcb:SetPoint("LEFT", frame.Health, "RIGHT") --attach to immediate right of health bar to start
    hcb:Hide() --hide it for now

    frame.HealCommBar = hcb

	local o = frame.PostUpdateHealth
	frame.PostUpdateHealth = function(...)
		if o then o(...) end
        local name, server = UnitName(frame.unit)
        if server then name = strjoin("-",name,server) end
        updateHealCommBar(frame, name) --update the bar when unit's health is updated
	end
end

--hook into all existing frames
for i, frame in ipairs(oUF.objects) do hook(frame) end

--hook into new frames as they're created
oUF:RegisterInitCallback(hook)

--set up LibHealComm callbacks
function oUF_HealComm:HealComm_HealStarted(event, casterGUID, spellID, healType, endTime, ...)
    updateHealCommBars(...)
end

function oUF_HealComm:HealComm_HealUpdated(event, casterGUID, spellID, healType, endTime, ...)
    updateHealCommBars(...)
end

function oUF_HealComm:HealComm_HealDelayed(event, casterGUID, spellID, healType, endTime, ...)
    updateHealCommBars(...)
end

function oUF_HealComm:HealComm_HealStopped(event, casterGUID, spellID, healType, interrupted, ...)
    updateHealCommBars(...)
end

function oUF_HealComm:HealComm_ModifierChanged(event, guid)
    updateHealCommBars(guid)
end

healcomm.RegisterCallback(oUF_HealComm, "HealComm_HealStarted")
healcomm.RegisterCallback(oUF_HealComm, "HealComm_HealStopped")
healcomm.RegisterCallback(oUF_HealComm, "HealComm_HealDelayed")
healcomm.RegisterCallback(oUF_HealComm, "HealComm_HealUpdated")
healcomm.RegisterCallback(oUF_HealComm, "HealComm_ModifierChanged")

